package cn.ibizlab.businesscentral.core.odoo_account.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_account.domain.Account_tax;
import cn.ibizlab.businesscentral.core.odoo_account.filter.Account_taxSearchContext;
import cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_taxService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_account.mapper.Account_taxMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[税率] 服务对象接口实现
 */
@Slf4j
@Service("Account_taxServiceImpl")
public class Account_taxServiceImpl extends ServiceImpl<Account_taxMapper, Account_tax> implements IAccount_taxService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_fiscal_position_taxService accountFiscalPositionTaxService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_invoice_taxService accountInvoiceTaxService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_move_lineService accountMoveLineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_reconcile_modelService accountReconcileModelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_companyService resCompanyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_accountService accountAccountService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_tax_groupService accountTaxGroupService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Account_tax et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Account_tax> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Account_tax et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Account_tax> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Account_tax get(Long key) {
        Account_tax et = getById(key);
        if(et==null){
            et=new Account_tax();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Account_tax et){
        //实体关系[DER1N_ACCOUNT_TAX__ACCOUNT_ACCOUNT__ACCOUNT_ID]
        if(!ObjectUtils.isEmpty(et.getAccountId())){
            cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account odooAccount=et.getOdooAccount();
            if(ObjectUtils.isEmpty(odooAccount)){
                cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account majorEntity=accountAccountService.get(et.getAccountId());
                et.setOdooAccount(majorEntity);
                odooAccount=majorEntity;
            }
            et.setAccountIdText(odooAccount.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__ACCOUNT_ACCOUNT__CASH_BASIS_ACCOUNT_ID]
        if(!ObjectUtils.isEmpty(et.getCashBasisAccountId())){
            cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account odooCashBasisAccount=et.getOdooCashBasisAccount();
            if(ObjectUtils.isEmpty(odooCashBasisAccount)){
                cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account majorEntity=accountAccountService.get(et.getCashBasisAccountId());
                et.setOdooCashBasisAccount(majorEntity);
                odooCashBasisAccount=majorEntity;
            }
            et.setCashBasisAccountIdText(odooCashBasisAccount.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__ACCOUNT_ACCOUNT__CASH_BASIS_BASE_ACCOUNT_ID]
        if(!ObjectUtils.isEmpty(et.getCashBasisBaseAccountId())){
            cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account odooCashBasisBaseAccount=et.getOdooCashBasisBaseAccount();
            if(ObjectUtils.isEmpty(odooCashBasisBaseAccount)){
                cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account majorEntity=accountAccountService.get(et.getCashBasisBaseAccountId());
                et.setOdooCashBasisBaseAccount(majorEntity);
                odooCashBasisBaseAccount=majorEntity;
            }
            et.setCashBasisBaseAccountIdText(odooCashBasisBaseAccount.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__ACCOUNT_ACCOUNT__REFUND_ACCOUNT_ID]
        if(!ObjectUtils.isEmpty(et.getRefundAccountId())){
            cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account odooRefundAccount=et.getOdooRefundAccount();
            if(ObjectUtils.isEmpty(odooRefundAccount)){
                cn.ibizlab.businesscentral.core.odoo_account.domain.Account_account majorEntity=accountAccountService.get(et.getRefundAccountId());
                et.setOdooRefundAccount(majorEntity);
                odooRefundAccount=majorEntity;
            }
            et.setRefundAccountIdText(odooRefundAccount.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__ACCOUNT_TAX_GROUP__TAX_GROUP_ID]
        if(!ObjectUtils.isEmpty(et.getTaxGroupId())){
            cn.ibizlab.businesscentral.core.odoo_account.domain.Account_tax_group odooTaxGroup=et.getOdooTaxGroup();
            if(ObjectUtils.isEmpty(odooTaxGroup)){
                cn.ibizlab.businesscentral.core.odoo_account.domain.Account_tax_group majorEntity=accountTaxGroupService.get(et.getTaxGroupId());
                et.setOdooTaxGroup(majorEntity);
                odooTaxGroup=majorEntity;
            }
            et.setTaxGroupIdText(odooTaxGroup.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__RES_COMPANY__COMPANY_ID]
        if(!ObjectUtils.isEmpty(et.getCompanyId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_company odooCompany=et.getOdooCompany();
            if(ObjectUtils.isEmpty(odooCompany)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_company majorEntity=resCompanyService.get(et.getCompanyId());
                et.setOdooCompany(majorEntity);
                odooCompany=majorEntity;
            }
            et.setHideTaxExigibility(odooCompany.getTaxExigibility());
            et.setCompanyIdText(odooCompany.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_ACCOUNT_TAX__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Account_tax> getAccountTaxByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Account_tax> getAccountTaxByEntities(List<Account_tax> entities) {
        List ids =new ArrayList();
        for(Account_tax entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



