package cn.ibizlab.businesscentral.core.odoo_fleet.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_cost;
import cn.ibizlab.businesscentral.core.odoo_fleet.filter.Fleet_vehicle_costSearchContext;
import cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_costService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_fleet.mapper.Fleet_vehicle_costMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[车辆相关的费用] 服务对象接口实现
 */
@Slf4j
@Service("Fleet_vehicle_costServiceImpl")
public class Fleet_vehicle_costServiceImpl extends ServiceImpl<Fleet_vehicle_costMapper, Fleet_vehicle_cost> implements IFleet_vehicle_costService {


    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_costService fleetVehicleCostService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_log_contractService fleetVehicleLogContractService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_log_fuelService fleetVehicleLogFuelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_log_servicesService fleetVehicleLogServicesService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_service_typeService fleetServiceTypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_odometerService fleetVehicleOdometerService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicleService fleetVehicleService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Fleet_vehicle_cost et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Fleet_vehicle_cost> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Fleet_vehicle_cost et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Fleet_vehicle_cost> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Fleet_vehicle_cost get(Long key) {
        Fleet_vehicle_cost et = getById(key);
        if(et==null){
            et=new Fleet_vehicle_cost();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Fleet_vehicle_cost et){
        //实体关系[DER1N_FLEET_VEHICLE_COST__FLEET_SERVICE_TYPE__COST_SUBTYPE_ID]
        if(!ObjectUtils.isEmpty(et.getCostSubtypeId())){
            cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_service_type odooCostSubtype=et.getOdooCostSubtype();
            if(ObjectUtils.isEmpty(odooCostSubtype)){
                cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_service_type majorEntity=fleetServiceTypeService.get(et.getCostSubtypeId());
                et.setOdooCostSubtype(majorEntity);
                odooCostSubtype=majorEntity;
            }
            et.setCostSubtypeIdText(odooCostSubtype.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_COST__FLEET_VEHICLE_COST__PARENT_ID]
        if(!ObjectUtils.isEmpty(et.getParentId())){
            cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_cost odooParent=et.getOdooParent();
            if(ObjectUtils.isEmpty(odooParent)){
                cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_cost majorEntity=fleetVehicleCostService.get(et.getParentId());
                et.setOdooParent(majorEntity);
                odooParent=majorEntity;
            }
            et.setParentIdText(odooParent.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_COST__FLEET_VEHICLE_LOG_CONTRACT__CONTRACT_ID]
        if(!ObjectUtils.isEmpty(et.getContractId())){
            cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_log_contract odooContract=et.getOdooContract();
            if(ObjectUtils.isEmpty(odooContract)){
                cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_log_contract majorEntity=fleetVehicleLogContractService.get(et.getContractId());
                et.setOdooContract(majorEntity);
                odooContract=majorEntity;
            }
            et.setContractIdText(odooContract.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_COST__FLEET_VEHICLE_ODOMETER__ODOMETER_ID]
        if(!ObjectUtils.isEmpty(et.getOdometerId())){
            cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_odometer odooOdometer=et.getOdooOdometer();
            if(ObjectUtils.isEmpty(odooOdometer)){
                cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_odometer majorEntity=fleetVehicleOdometerService.get(et.getOdometerId());
                et.setOdooOdometer(majorEntity);
                odooOdometer=majorEntity;
            }
            et.setOdometerIdText(odooOdometer.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_COST__FLEET_VEHICLE__VEHICLE_ID]
        if(!ObjectUtils.isEmpty(et.getVehicleId())){
            cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle odooVehicle=et.getOdooVehicle();
            if(ObjectUtils.isEmpty(odooVehicle)){
                cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle majorEntity=fleetVehicleService.get(et.getVehicleId());
                et.setOdooVehicle(majorEntity);
                odooVehicle=majorEntity;
            }
            et.setName(odooVehicle.getName());
            et.setOdometerUnit(odooVehicle.getOdometerUnit());
        }
        //实体关系[DER1N_FLEET_VEHICLE_COST__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_COST__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Fleet_vehicle_cost> getFleetVehicleCostByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Fleet_vehicle_cost> getFleetVehicleCostByEntities(List<Fleet_vehicle_cost> entities) {
        List ids =new ArrayList();
        for(Fleet_vehicle_cost entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



