package cn.ibizlab.businesscentral.core.odoo_fleet.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_log_contract;
import cn.ibizlab.businesscentral.core.odoo_fleet.filter.Fleet_vehicle_log_contractSearchContext;
import cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_log_contractService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_fleet.mapper.Fleet_vehicle_log_contractMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[车辆合同信息] 服务对象接口实现
 */
@Slf4j
@Service("Fleet_vehicle_log_contractServiceImpl")
public class Fleet_vehicle_log_contractServiceImpl extends ServiceImpl<Fleet_vehicle_log_contractMapper, Fleet_vehicle_log_contract> implements IFleet_vehicle_log_contractService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_fleet.service.IFleet_vehicle_costService fleetVehicleCostService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_partnerService resPartnerService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Fleet_vehicle_log_contract et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Fleet_vehicle_log_contract> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Fleet_vehicle_log_contract et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Fleet_vehicle_log_contract> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Fleet_vehicle_log_contract get(Long key) {
        Fleet_vehicle_log_contract et = getById(key);
        if(et==null){
            et=new Fleet_vehicle_log_contract();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Fleet_vehicle_log_contract et){
        //实体关系[DER1N_FLEET_VEHICLE_LOG_CONTRACT__FLEET_VEHICLE_COST__COST_ID]
        if(!ObjectUtils.isEmpty(et.getCostId())){
            cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_cost odooCost=et.getOdooCost();
            if(ObjectUtils.isEmpty(odooCost)){
                cn.ibizlab.businesscentral.core.odoo_fleet.domain.Fleet_vehicle_cost majorEntity=fleetVehicleCostService.get(et.getCostId());
                et.setOdooCost(majorEntity);
                odooCost=majorEntity;
            }
            et.setVehicleId(odooCost.getVehicleId());
            et.setAutoGenerated(odooCost.getAutoGenerated());
            et.setOdometerId(odooCost.getOdometerId());
            et.setCostType(odooCost.getCostType());
            et.setParentId(odooCost.getParentId());
            et.setCostSubtypeId(odooCost.getCostSubtypeId());
            et.setCostAmount(odooCost.getAmount());
            et.setAmount(odooCost.getAmount());
            et.setCostIdText(odooCost.getName());
            et.setDescription(odooCost.getDescription());
            et.setDate(odooCost.getDate());
            et.setOdometerUnit(odooCost.getOdometerUnit());
            et.setContractId(odooCost.getContractId());
        }
        //实体关系[DER1N_FLEET_VEHICLE_LOG_CONTRACT__RES_PARTNER__INSURER_ID]
        if(!ObjectUtils.isEmpty(et.getInsurerId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner odooInsurer=et.getOdooInsurer();
            if(ObjectUtils.isEmpty(odooInsurer)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner majorEntity=resPartnerService.get(et.getInsurerId());
                et.setOdooInsurer(majorEntity);
                odooInsurer=majorEntity;
            }
            et.setInsurerIdText(odooInsurer.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_LOG_CONTRACT__RES_PARTNER__PURCHASER_ID]
        if(!ObjectUtils.isEmpty(et.getPurchaserId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner odooPurchaser=et.getOdooPurchaser();
            if(ObjectUtils.isEmpty(odooPurchaser)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner majorEntity=resPartnerService.get(et.getPurchaserId());
                et.setOdooPurchaser(majorEntity);
                odooPurchaser=majorEntity;
            }
            et.setPurchaserIdText(odooPurchaser.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_LOG_CONTRACT__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_LOG_CONTRACT__RES_USERS__USER_ID]
        if(!ObjectUtils.isEmpty(et.getUserId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooUser=et.getOdooUser();
            if(ObjectUtils.isEmpty(odooUser)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getUserId());
                et.setOdooUser(majorEntity);
                odooUser=majorEntity;
            }
            et.setUserIdText(odooUser.getName());
        }
        //实体关系[DER1N_FLEET_VEHICLE_LOG_CONTRACT__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Fleet_vehicle_log_contract> getFleetVehicleLogContractByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Fleet_vehicle_log_contract> getFleetVehicleLogContractByEntities(List<Fleet_vehicle_log_contract> entities) {
        List ids =new ArrayList();
        for(Fleet_vehicle_log_contract entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



