package cn.ibizlab.businesscentral.core.odoo_gamification.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_challenge;
import cn.ibizlab.businesscentral.core.odoo_gamification.filter.Gamification_challengeSearchContext;
import cn.ibizlab.businesscentral.core.odoo_gamification.service.IGamification_challengeService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_gamification.mapper.Gamification_challengeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[游戏化挑战] 服务对象接口实现
 */
@Slf4j
@Service("Gamification_challengeServiceImpl")
public class Gamification_challengeServiceImpl extends ServiceImpl<Gamification_challengeMapper, Gamification_challenge> implements IGamification_challengeService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_gamification.service.IGamification_badge_userService gamificationBadgeUserService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_gamification.service.IGamification_challenge_lineService gamificationChallengeLineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_gamification.service.IGamification_goalService gamificationGoalService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_gamification.service.IGamification_badgeService gamificationBadgeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mail.service.IMail_channelService mailChannelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mail.service.IMail_templateService mailTemplateService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Gamification_challenge et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Gamification_challenge> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Gamification_challenge et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Gamification_challenge> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Gamification_challenge get(Long key) {
        Gamification_challenge et = getById(key);
        if(et==null){
            et=new Gamification_challenge();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Gamification_challenge et){
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__GAMIFICATION_BADGE__REWARD_FIRST_ID]
        if(!ObjectUtils.isEmpty(et.getRewardFirstId())){
            cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge odooRewardFirst=et.getOdooRewardFirst();
            if(ObjectUtils.isEmpty(odooRewardFirst)){
                cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge majorEntity=gamificationBadgeService.get(et.getRewardFirstId());
                et.setOdooRewardFirst(majorEntity);
                odooRewardFirst=majorEntity;
            }
            et.setRewardFirstIdText(odooRewardFirst.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__GAMIFICATION_BADGE__REWARD_ID]
        if(!ObjectUtils.isEmpty(et.getRewardId())){
            cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge odooReward=et.getOdooReward();
            if(ObjectUtils.isEmpty(odooReward)){
                cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge majorEntity=gamificationBadgeService.get(et.getRewardId());
                et.setOdooReward(majorEntity);
                odooReward=majorEntity;
            }
            et.setRewardIdText(odooReward.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__GAMIFICATION_BADGE__REWARD_SECOND_ID]
        if(!ObjectUtils.isEmpty(et.getRewardSecondId())){
            cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge odooRewardSecond=et.getOdooRewardSecond();
            if(ObjectUtils.isEmpty(odooRewardSecond)){
                cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge majorEntity=gamificationBadgeService.get(et.getRewardSecondId());
                et.setOdooRewardSecond(majorEntity);
                odooRewardSecond=majorEntity;
            }
            et.setRewardSecondIdText(odooRewardSecond.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__GAMIFICATION_BADGE__REWARD_THIRD_ID]
        if(!ObjectUtils.isEmpty(et.getRewardThirdId())){
            cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge odooRewardThird=et.getOdooRewardThird();
            if(ObjectUtils.isEmpty(odooRewardThird)){
                cn.ibizlab.businesscentral.core.odoo_gamification.domain.Gamification_badge majorEntity=gamificationBadgeService.get(et.getRewardThirdId());
                et.setOdooRewardThird(majorEntity);
                odooRewardThird=majorEntity;
            }
            et.setRewardThirdIdText(odooRewardThird.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__MAIL_CHANNEL__REPORT_MESSAGE_GROUP_ID]
        if(!ObjectUtils.isEmpty(et.getReportMessageGroupId())){
            cn.ibizlab.businesscentral.core.odoo_mail.domain.Mail_channel odooReportMessageGroup=et.getOdooReportMessageGroup();
            if(ObjectUtils.isEmpty(odooReportMessageGroup)){
                cn.ibizlab.businesscentral.core.odoo_mail.domain.Mail_channel majorEntity=mailChannelService.get(et.getReportMessageGroupId());
                et.setOdooReportMessageGroup(majorEntity);
                odooReportMessageGroup=majorEntity;
            }
            et.setReportMessageGroupIdText(odooReportMessageGroup.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__MAIL_TEMPLATE__REPORT_TEMPLATE_ID]
        if(!ObjectUtils.isEmpty(et.getReportTemplateId())){
            cn.ibizlab.businesscentral.core.odoo_mail.domain.Mail_template odooReportTemplate=et.getOdooReportTemplate();
            if(ObjectUtils.isEmpty(odooReportTemplate)){
                cn.ibizlab.businesscentral.core.odoo_mail.domain.Mail_template majorEntity=mailTemplateService.get(et.getReportTemplateId());
                et.setOdooReportTemplate(majorEntity);
                odooReportTemplate=majorEntity;
            }
            et.setReportTemplateIdText(odooReportTemplate.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__RES_USERS__MANAGER_ID]
        if(!ObjectUtils.isEmpty(et.getManagerId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooManager=et.getOdooManager();
            if(ObjectUtils.isEmpty(odooManager)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getManagerId());
                et.setOdooManager(majorEntity);
                odooManager=majorEntity;
            }
            et.setManagerIdText(odooManager.getName());
        }
        //实体关系[DER1N_GAMIFICATION_CHALLENGE__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Gamification_challenge> getGamificationChallengeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Gamification_challenge> getGamificationChallengeByEntities(List<Gamification_challenge> entities) {
        List ids =new ArrayList();
        for(Gamification_challenge entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



