package cn.ibizlab.businesscentral.core.odoo_im_livechat.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_im_livechat.domain.Im_livechat_report_channel;
import cn.ibizlab.businesscentral.core.odoo_im_livechat.filter.Im_livechat_report_channelSearchContext;
import cn.ibizlab.businesscentral.core.odoo_im_livechat.service.IIm_livechat_report_channelService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_im_livechat.mapper.Im_livechat_report_channelMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[实时聊天支持频道报告] 服务对象接口实现
 */
@Slf4j
@Service("Im_livechat_report_channelServiceImpl")
public class Im_livechat_report_channelServiceImpl extends ServiceImpl<Im_livechat_report_channelMapper, Im_livechat_report_channel> implements IIm_livechat_report_channelService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_im_livechat.service.IIm_livechat_channelService imLivechatChannelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mail.service.IMail_channelService mailChannelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_partnerService resPartnerService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Im_livechat_report_channel et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Im_livechat_report_channel> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Im_livechat_report_channel et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Im_livechat_report_channel> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Im_livechat_report_channel get(Long key) {
        Im_livechat_report_channel et = getById(key);
        if(et==null){
            et=new Im_livechat_report_channel();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Im_livechat_report_channel et){
        //实体关系[DER1N_IM_LIVECHAT_REPORT_CHANNEL__IM_LIVECHAT_CHANNEL__LIVECHAT_CHANNEL_ID]
        if(!ObjectUtils.isEmpty(et.getLivechatChannelId())){
            cn.ibizlab.businesscentral.core.odoo_im_livechat.domain.Im_livechat_channel odooLivechatChannel=et.getOdooLivechatChannel();
            if(ObjectUtils.isEmpty(odooLivechatChannel)){
                cn.ibizlab.businesscentral.core.odoo_im_livechat.domain.Im_livechat_channel majorEntity=imLivechatChannelService.get(et.getLivechatChannelId());
                et.setOdooLivechatChannel(majorEntity);
                odooLivechatChannel=majorEntity;
            }
            et.setLivechatChannelIdText(odooLivechatChannel.getName());
        }
        //实体关系[DER1N_IM_LIVECHAT_REPORT_CHANNEL__MAIL_CHANNEL__CHANNEL_ID]
        if(!ObjectUtils.isEmpty(et.getChannelId())){
            cn.ibizlab.businesscentral.core.odoo_mail.domain.Mail_channel odooChannel=et.getOdooChannel();
            if(ObjectUtils.isEmpty(odooChannel)){
                cn.ibizlab.businesscentral.core.odoo_mail.domain.Mail_channel majorEntity=mailChannelService.get(et.getChannelId());
                et.setOdooChannel(majorEntity);
                odooChannel=majorEntity;
            }
            et.setChannelIdText(odooChannel.getName());
        }
        //实体关系[DER1N_IM_LIVECHAT_REPORT_CHANNEL__RES_PARTNER__PARTNER_ID]
        if(!ObjectUtils.isEmpty(et.getPartnerId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner odooPartner=et.getOdooPartner();
            if(ObjectUtils.isEmpty(odooPartner)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner majorEntity=resPartnerService.get(et.getPartnerId());
                et.setOdooPartner(majorEntity);
                odooPartner=majorEntity;
            }
            et.setPartnerIdText(odooPartner.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }




}



