package cn.ibizlab.businesscentral.core.odoo_mro.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_order;
import cn.ibizlab.businesscentral.core.odoo_mro.filter.Mro_orderSearchContext;
import cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_orderService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_mro.mapper.Mro_orderMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[Maintenance Order] 服务对象接口实现
 */
@Slf4j
@Service("Mro_orderServiceImpl")
public class Mro_orderServiceImpl extends ServiceImpl<Mro_orderMapper, Mro_order> implements IMro_orderService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_order_parts_lineService mroOrderPartsLineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_asset.service.IAsset_assetService assetAssetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_requestService mroRequestService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_taskService mroTaskService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_workorderService mroWorkorderService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_companyService resCompanyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Mro_order et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Mro_order> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Mro_order et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Mro_order> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Mro_order get(Long key) {
        Mro_order et = getById(key);
        if(et==null){
            et=new Mro_order();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Mro_order et){
        //实体关系[DER1N_MRO_ORDER__ASSET_ASSET__ASSET_ID]
        if(!ObjectUtils.isEmpty(et.getAssetId())){
            cn.ibizlab.businesscentral.core.odoo_asset.domain.Asset_asset odooAsset=et.getOdooAsset();
            if(ObjectUtils.isEmpty(odooAsset)){
                cn.ibizlab.businesscentral.core.odoo_asset.domain.Asset_asset majorEntity=assetAssetService.get(et.getAssetId());
                et.setOdooAsset(majorEntity);
                odooAsset=majorEntity;
            }
            et.setAssetIdText(odooAsset.getName());
        }
        //实体关系[DER1N_MRO_ORDER__MRO_REQUEST__REQUEST_ID]
        if(!ObjectUtils.isEmpty(et.getRequestId())){
            cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_request odooRequest=et.getOdooRequest();
            if(ObjectUtils.isEmpty(odooRequest)){
                cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_request majorEntity=mroRequestService.get(et.getRequestId());
                et.setOdooRequest(majorEntity);
                odooRequest=majorEntity;
            }
            et.setRequestIdText(odooRequest.getName());
        }
        //实体关系[DER1N_MRO_ORDER__MRO_TASK__TASK_ID]
        if(!ObjectUtils.isEmpty(et.getTaskId())){
            cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_task odooTask=et.getOdooTask();
            if(ObjectUtils.isEmpty(odooTask)){
                cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_task majorEntity=mroTaskService.get(et.getTaskId());
                et.setOdooTask(majorEntity);
                odooTask=majorEntity;
            }
            et.setTaskIdText(odooTask.getName());
        }
        //实体关系[DER1N_MRO_ORDER__MRO_WORKORDER__WO_ID]
        if(!ObjectUtils.isEmpty(et.getWoId())){
            cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_workorder odooWo=et.getOdooWo();
            if(ObjectUtils.isEmpty(odooWo)){
                cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_workorder majorEntity=mroWorkorderService.get(et.getWoId());
                et.setOdooWo(majorEntity);
                odooWo=majorEntity;
            }
            et.setWoIdText(odooWo.getName());
        }
        //实体关系[DER1N_MRO_ORDER__RES_COMPANY__COMPANY_ID]
        if(!ObjectUtils.isEmpty(et.getCompanyId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_company odooCompany=et.getOdooCompany();
            if(ObjectUtils.isEmpty(odooCompany)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_company majorEntity=resCompanyService.get(et.getCompanyId());
                et.setOdooCompany(majorEntity);
                odooCompany=majorEntity;
            }
            et.setCompanyIdText(odooCompany.getName());
        }
        //实体关系[DER1N_MRO_ORDER__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_MRO_ORDER__RES_USERS__USER_ID]
        if(!ObjectUtils.isEmpty(et.getUserId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooUser=et.getOdooUser();
            if(ObjectUtils.isEmpty(odooUser)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getUserId());
                et.setOdooUser(majorEntity);
                odooUser=majorEntity;
            }
            et.setUserIdText(odooUser.getName());
        }
        //实体关系[DER1N_MRO_ORDER__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Mro_order> getMroOrderByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Mro_order> getMroOrderByEntities(List<Mro_order> entities) {
        List ids =new ArrayList();
        for(Mro_order entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



