package cn.ibizlab.businesscentral.core.odoo_mro.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_pm_rule_line;
import cn.ibizlab.businesscentral.core.odoo_mro.filter.Mro_pm_rule_lineSearchContext;
import cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_pm_rule_lineService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_mro.mapper.Mro_pm_rule_lineMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[Rule for Task] 服务对象接口实现
 */
@Slf4j
@Service("Mro_pm_rule_lineServiceImpl")
public class Mro_pm_rule_lineServiceImpl extends ServiceImpl<Mro_pm_rule_lineMapper, Mro_pm_rule_line> implements IMro_pm_rule_lineService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_pm_meter_intervalService mroPmMeterIntervalService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_pm_ruleService mroPmRuleService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_mro.service.IMro_taskService mroTaskService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Mro_pm_rule_line et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Mro_pm_rule_line> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Mro_pm_rule_line et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Mro_pm_rule_line> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Mro_pm_rule_line get(Long key) {
        Mro_pm_rule_line et = getById(key);
        if(et==null){
            et=new Mro_pm_rule_line();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Mro_pm_rule_line et){
        //实体关系[DER1N_MRO_PM_RULE_LINE__MRO_PM_METER_INTERVAL__METER_INTERVAL_ID]
        if(!ObjectUtils.isEmpty(et.getMeterIntervalId())){
            cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_pm_meter_interval odooMeterInterval=et.getOdooMeterInterval();
            if(ObjectUtils.isEmpty(odooMeterInterval)){
                cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_pm_meter_interval majorEntity=mroPmMeterIntervalService.get(et.getMeterIntervalId());
                et.setOdooMeterInterval(majorEntity);
                odooMeterInterval=majorEntity;
            }
            et.setMeterIntervalIdText(odooMeterInterval.getName());
        }
        //实体关系[DER1N_MRO_PM_RULE_LINE__MRO_PM_RULE__PM_RULE_ID]
        if(!ObjectUtils.isEmpty(et.getPmRuleId())){
            cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_pm_rule odooPmRule=et.getOdooPmRule();
            if(ObjectUtils.isEmpty(odooPmRule)){
                cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_pm_rule majorEntity=mroPmRuleService.get(et.getPmRuleId());
                et.setOdooPmRule(majorEntity);
                odooPmRule=majorEntity;
            }
            et.setPmRuleIdText(odooPmRule.getName());
        }
        //实体关系[DER1N_MRO_PM_RULE_LINE__MRO_TASK__TASK_ID]
        if(!ObjectUtils.isEmpty(et.getTaskId())){
            cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_task odooTask=et.getOdooTask();
            if(ObjectUtils.isEmpty(odooTask)){
                cn.ibizlab.businesscentral.core.odoo_mro.domain.Mro_task majorEntity=mroTaskService.get(et.getTaskId());
                et.setOdooTask(majorEntity);
                odooTask=majorEntity;
            }
            et.setTaskIdText(odooTask.getName());
        }
        //实体关系[DER1N_MRO_PM_RULE_LINE__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_MRO_PM_RULE_LINE__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Mro_pm_rule_line> getMroPmRuleLineByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Mro_pm_rule_line> getMroPmRuleLineByEntities(List<Mro_pm_rule_line> entities) {
        List ids =new ArrayList();
        for(Mro_pm_rule_line entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



