package cn.ibizlab.businesscentral.core.odoo_payment.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.businesscentral.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.businesscentral.core.odoo_payment.domain.Payment_transaction;
import cn.ibizlab.businesscentral.core.odoo_payment.filter.Payment_transactionSearchContext;
import cn.ibizlab.businesscentral.core.odoo_payment.service.IPayment_transactionService;

import cn.ibizlab.businesscentral.util.helper.CachedBeanCopier;
import cn.ibizlab.businesscentral.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.businesscentral.core.odoo_payment.mapper.Payment_transactionMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[付款交易] 服务对象接口实现
 */
@Slf4j
@Service("Payment_transactionServiceImpl")
public class Payment_transactionServiceImpl extends ServiceImpl<Payment_transactionMapper, Payment_transaction> implements IPayment_transactionService {

    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_account.service.IAccount_paymentService accountPaymentService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_payment.service.IPayment_acquirerService paymentAcquirerService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_payment.service.IPayment_tokenService paymentTokenService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_countryService resCountryService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_currencyService resCurrencyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_partnerService resPartnerService;
    @Autowired
    @Lazy
    protected cn.ibizlab.businesscentral.core.odoo_base.service.IRes_usersService resUsersService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(Payment_transaction et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<Payment_transaction> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(Payment_transaction et) {
        fillParentData(et);
         if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("id",et.getId())))
            return false;
        CachedBeanCopier.copy(get(et.getId()),et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<Payment_transaction> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(Long key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<Long> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public Payment_transaction get(Long key) {
        Payment_transaction et = getById(key);
        if(et==null){
            et=new Payment_transaction();
            et.setId(key);
        }
        else{
        }
        return et;
    }





    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(Payment_transaction et){
        //实体关系[DER1N_PAYMENT_TRANSACTION__ACCOUNT_PAYMENT__PAYMENT_ID]
        if(!ObjectUtils.isEmpty(et.getPaymentId())){
            cn.ibizlab.businesscentral.core.odoo_account.domain.Account_payment odooPayment=et.getOdooPayment();
            if(ObjectUtils.isEmpty(odooPayment)){
                cn.ibizlab.businesscentral.core.odoo_account.domain.Account_payment majorEntity=accountPaymentService.get(et.getPaymentId());
                et.setOdooPayment(majorEntity);
                odooPayment=majorEntity;
            }
            et.setPaymentIdText(odooPayment.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__PAYMENT_ACQUIRER__ACQUIRER_ID]
        if(!ObjectUtils.isEmpty(et.getAcquirerId())){
            cn.ibizlab.businesscentral.core.odoo_payment.domain.Payment_acquirer odooAcquirer=et.getOdooAcquirer();
            if(ObjectUtils.isEmpty(odooAcquirer)){
                cn.ibizlab.businesscentral.core.odoo_payment.domain.Payment_acquirer majorEntity=paymentAcquirerService.get(et.getAcquirerId());
                et.setOdooAcquirer(majorEntity);
                odooAcquirer=majorEntity;
            }
            et.setProvider(odooAcquirer.getProvider());
            et.setAcquirerIdText(odooAcquirer.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__PAYMENT_TOKEN__PAYMENT_TOKEN_ID]
        if(!ObjectUtils.isEmpty(et.getPaymentTokenId())){
            cn.ibizlab.businesscentral.core.odoo_payment.domain.Payment_token odooPaymentToken=et.getOdooPaymentToken();
            if(ObjectUtils.isEmpty(odooPaymentToken)){
                cn.ibizlab.businesscentral.core.odoo_payment.domain.Payment_token majorEntity=paymentTokenService.get(et.getPaymentTokenId());
                et.setOdooPaymentToken(majorEntity);
                odooPaymentToken=majorEntity;
            }
            et.setPaymentTokenIdText(odooPaymentToken.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__RES_COUNTRY__PARTNER_COUNTRY_ID]
        if(!ObjectUtils.isEmpty(et.getPartnerCountryId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_country odooPartnerCountry=et.getOdooPartnerCountry();
            if(ObjectUtils.isEmpty(odooPartnerCountry)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_country majorEntity=resCountryService.get(et.getPartnerCountryId());
                et.setOdooPartnerCountry(majorEntity);
                odooPartnerCountry=majorEntity;
            }
            et.setPartnerCountryIdText(odooPartnerCountry.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__RES_CURRENCY__CURRENCY_ID]
        if(!ObjectUtils.isEmpty(et.getCurrencyId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_currency odooCurrency=et.getOdooCurrency();
            if(ObjectUtils.isEmpty(odooCurrency)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_currency majorEntity=resCurrencyService.get(et.getCurrencyId());
                et.setOdooCurrency(majorEntity);
                odooCurrency=majorEntity;
            }
            et.setCurrencyIdText(odooCurrency.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__RES_PARTNER__PARTNER_ID]
        if(!ObjectUtils.isEmpty(et.getPartnerId())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner odooPartner=et.getOdooPartner();
            if(ObjectUtils.isEmpty(odooPartner)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_partner majorEntity=resPartnerService.get(et.getPartnerId());
                et.setOdooPartner(majorEntity);
                odooPartner=majorEntity;
            }
            et.setPartnerIdText(odooPartner.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__RES_USERS__CREATE_UID]
        if(!ObjectUtils.isEmpty(et.getCreateUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooCreate=et.getOdooCreate();
            if(ObjectUtils.isEmpty(odooCreate)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getCreateUid());
                et.setOdooCreate(majorEntity);
                odooCreate=majorEntity;
            }
            et.setCreateUidText(odooCreate.getName());
        }
        //实体关系[DER1N_PAYMENT_TRANSACTION__RES_USERS__WRITE_UID]
        if(!ObjectUtils.isEmpty(et.getWriteUid())){
            cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users odooWrite=et.getOdooWrite();
            if(ObjectUtils.isEmpty(odooWrite)){
                cn.ibizlab.businesscentral.core.odoo_base.domain.Res_users majorEntity=resUsersService.get(et.getWriteUid());
                et.setOdooWrite(majorEntity);
                odooWrite=majorEntity;
            }
            et.setWriteUidText(odooWrite.getName());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<Payment_transaction> getPaymentTransactionByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<Payment_transaction> getPaymentTransactionByEntities(List<Payment_transaction> entities) {
        List ids =new ArrayList();
        for(Payment_transaction entity : entities){
            Serializable id=entity.getId();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }



}



